# (c) 2020 MPIB <https://www.mpib-berlin.mpg.de/>,
#
# This file is part of castellum-scheduler.
#
# castellum-scheduler is free software; you can redistribute it and/or
# modify it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation; either version 3 of the
# License, or (at your option) any later version.
#
# Castellum is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with Castellum. If not, see
# <http://www.gnu.org/licenses/>.

from collections import OrderedDict

from django.contrib.auth.mixins import LoginRequiredMixin
from django.contrib import messages
from django.shortcuts import get_object_or_404
from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from django.views.generic import CreateView
from django.views.generic import DeleteView
from django.views.generic import ListView
from django.views.generic import UpdateView

from .forms import ScheduleForm
from .models import Invitation
from .models import Schedule


class ScheduleListView(LoginRequiredMixin, ListView):
    model = Schedule
    paginate_by = 2


class ScheduleFormMixin(LoginRequiredMixin):
    model = Schedule
    form_class = ScheduleForm

    def get_success_url(self):
        return reverse('schedule-update', args=[self.object.pk])

    def form_valid(self, form, *args):
        messages.success(self.request, _('Data has been saved.'))
        return super().form_valid(form, *args)


class ScheduleCreateView(ScheduleFormMixin, CreateView):
    pass


class ScheduleUpdateView(ScheduleFormMixin, UpdateView):
    pass


class ScheduleDeleteView(LoginRequiredMixin, DeleteView):
    model = Schedule

    def get_success_url(self):
        return reverse('index')


class InvitationUpdateView(UpdateView):
    model = Invitation
    slug_field = 'token'
    slug_url_kwarg = 'token'
    fields = ['timeslot']

    def get_object(self):
        return get_object_or_404(
            Invitation,
            schedule__pk=self.kwargs['schedule_pk'],
            token=self.kwargs['token'],
        )

    def get_success_url(self):
        return reverse('invitation', args=[self.object.schedule.pk, self.object.token])

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)

        timeslots = list(self.object.schedule.timeslot_set.all())

        dates = []
        times = []
        for timeslot in timeslots:
            dates.append(timeslot.date)
            times.append(timeslot.time)

        dates = sorted(set(dates))
        times = sorted(set(times))

        table = OrderedDict()
        for date in dates:
            table[date] = OrderedDict()
            for time in times:
                table[date][time] = 0
        for timeslot in timeslots:
            table[timeslot.date][timeslot.time] = timeslot.id

        context['table'] = {
            'timeslots': table,
            'times': times,
        }

        return context
